<?php
/**
 * AJAX handler to look up RVK data.
 *
 * PHP version 7
 *
 * Copyright (C) Villanova University 2018.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category finc\Rvktree\AjaxHandler
 * @package  AJAX
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Johannes Schultze <schultze@effective-webwork.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace finc\Rvktree\AjaxHandler;

use VuFind\DoiLinker\PluginManager;
use VuFind\XSLT\Import\VuFind;
use Zend\Mvc\Controller\Plugin\Params;

/**
 * AJAX handler to look up DOI data.
 *
 * @category finc\Rvktree\AjaxHandler
 * @package  AJAX
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Johannes Schultze <schultze@effective-webwork.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class Rvk extends \VuFind\AjaxHandler\AbstractBase
{
    use \VuFind\I18n\Translator\TranslatorAwareTrait;

    /**
     * DOI Linker Plugin Manager
     *
     * @var PluginManager
     */
    protected $pluginManager;

    /**
     * Rvktree configuration
     *
     * @var string
     */
    protected $config;
    
    const DEFAULT_API_URL = "http://rvk.uni-regensburg.de/api/json/children/";
    const DEFAULT_RVK_SECTIONS = ['A','B','CA - CK','CL - CZ','D','E','F','G','H','I','K','LA - LC','LD - LG','LD,LH - LO','LD,LP - LY','MA - ML','MN - MS','N','P','Q','R','U','V','W'];

    /**
     * Constructor
     *
     * @param PluginManager $pluginManager DOI Linker Plugin Manager
     * @param string        $resolver      DOI resolver configuration value
     */
    public function __construct(PluginManager $pluginManager, $config)
    {
        $this->pluginManager = $pluginManager;
        $this->config = $config;
    }

    /**
     * Handle a request.
     *
     * @param Params $params Parameter helper from controller
     *
     * @return array [response data, HTTP status code]
     */
    public function handleRequest(Params $params)
    {
        $result = array();

        $getNotation = $params->fromQuery('notation', '');

        if (!isset($getNotation) || $getNotation == '#') {

            $node = array();
            $rvkSections = $this->config['RVKWidget']['rvk'] ?? SELF::DEFAULT_RVK_SECTIONS;
            foreach($rvkSections as $rvk) {
                $rvkData = $this->getRvkData($rvk);
                $node['id'] = str_ireplace(' ', '_', $rvkData->node->notation);
                $node['text'] = $rvk.' :: '.$rvkData->node->benennung;
                $node['children'] = $rvkData->node->has_children !== 'no';
                $result[] = $node;
            }
        } else {
            $rvkData = $this->getRvkData(str_ireplace('_', ' ', $getNotation));
            if (isset($rvkData->node->children->node)) {
                foreach ($rvkData->node->children->node as $nodeData) {
                    $result[] = $this->getRvkNode($nodeData, $rvkData->node->notation);
                }
            }
        }

        return $this->formatResponse($result);
    }

    /**
     * Get RVK data for children of single note from RVK-API.
     *
     * @param $notation Current node.
     * @return mixed
     */
    private function getRvkData ($notation) {
        $apiUrl = $this->config['RVKWidget']['apiUrl'] ?? self::DEFAULT_API_URL;
        $jsonData = file_get_contents($apiUrl.urlencode($notation));
        return json_decode($jsonData);
    }

    /**
     * Get RVK data in jstree structure.
     *
     * @param $data Data of RVK node to be prepared.
     * @param null $parent Id of parent ot be used in tree contruction.
     * @return array
     */
    private function getRvkNode ($data, $parent = NULL) {
        /*
        JSON as data source for jstree (one node)
        {
            id          : "string"      // will be autogenerated if omitted
            text        : "string"      // node text
            icon        : "string"      // string for custom
            state       : {
                opened    : boolean     // is the node open
                disabled  : boolean     // is the node disabled
                selected  : boolean     // is the node selected
            },
            children    : []            // array of strings or objects
            li_attr     : {}            // attributes for the generated LI node
            a_attr      : {}            // attributes for the generated A node
        }
        */

        $result = array();

        if ($data) {
            $result['id'] = str_ireplace(' ', '_', $data->notation);
            $result['text'] = $data->notation.' :: '.$data->benennung;
            if ($parent) {
                $result['parent'] = str_ireplace(' ', '_', $parent);
            }
            if ($data->has_children) {
                $result['children'] = $data->has_children !== 'no';
            }
            $result['data']['searchLink'] = '<a href="/Search/Results?lookfor=%22'.urlencode($data->notation).'%22&type=rvkfglh">'.$this->translate('Show').'</a>';
        }

        return $result;
    }
}
